<?php
/**
 * @file
 * Contains form for customer setup.
 */
namespace Drupal\miniorange_2fa\Form;

use Drupal\user\Entity\User;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_2fa\MiniorangeUser;
use Drupal\miniorange_2fa\UsersAPIHandler;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\miniorange_2fa\MoAuthConstants;
use Drupal\miniorange_2fa\AuthenticationType;
use Drupal\miniorange_2fa\MiniorangeCustomerSetup;
use Drupal\miniorange_2fa\MiniorangeCustomerProfile;

/**
 * Customer setup form().
 */
class MoAuthCustomerSetup extends FormBase
{
  public function getFormId() {
    return 'miniorange_2fa_customer_setup';
  }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $user_obj = User::load(\Drupal::currentUser()->id());
        $user_id = $user_obj->id();
        $current_status = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_status');

        if ($current_status == 'VALIDATE_OTP') {

            $form['markup_top_2'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
            );

            $form['mo_auth_customer_otp_token'] = array(
                '#type' => 'textfield',
                '#title' => t('Please enter the OTP you received<span style="color: red">*</span>'),
                '#attributes' => array('autofocus' => 'true'),
                '#maxlength' => 6,
            );
            $form['mo_auth_customer_validate_otp_button'] = array(
                '#type' => 'submit',
                '#value' => t('Validate OTP'),
                '#submit' => array('::mo_auth_validate_otp_submit'),
            );
            $form['Mo_auth_customer_setup_resendotp'] = array(
                '#type' => 'submit',
                '#value' => t('Resend OTP'),
                '#submit' => array('::mo_auth_resend_otp'),
            );
            $form['Mo_auth_customer_setup_back'] = array(
                '#type' => 'submit',
                '#value' => t('Back'),
                '#submit' => array('::mo_auth_back'),
            );

            $form['main_layout_div_end'] = array(
                '#markup' => '<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br></div>',
            );

            MoAuthUtilities::AddsupportTab( $form, $form_state);

            return $form;

        } elseif ($current_status == 'PLUGIN_CONFIGURATION') {
            // Show customer configuration here.

            $utilities = new MoAuthUtilities();
            $custom_attribute = $utilities::get_users_custom_attribute($user_id);
            $user_email = $custom_attribute[0]->miniorange_registered_email;
            $customer = new MiniorangeCustomerProfile();
            $user_api_handler = new UsersAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $user_email, '', '', '');
            $response = $user_api_handler->get($miniorange_user);
            $authType = AuthenticationType::getAuthType($response->authType);

            $form['markup_top_2'] = array(
                '#markup' => '<div class="mo_saml_table_layout_1"><div id="Register_Section" class="mo_saml_table_layout mo_saml_container">'
            );

            $form['markup_top'] = array(
                '#markup' => '<div class="thankyou_message_on_register_tab">Thank you for registering with miniOrange</div>' . '<h4>Your Profile: </h4>'
            );

            $header = array('attribute' => array('data' => t('Attribute')), 'value' => array('data' => t('Value')));

            $options = array();
            $options[0] = array(
                'attribute' => '2 Factor Registered Email',
                'value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_email')
            );
            $options[1] = array(
                'attribute' => 'Activated 2nd Factor',
                'value' => $authType['name']
            );
            $options[2] = array(
                'attribute' => 'Xecurify Registered Email',
                'value' => $user_email
            );
            $options[3] = array(
                'attribute' => 'Customer ID',
                'value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_id')
            );
            $options[4] = array(
                'attribute' => 'API Key',
                'value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_api_key')
            );
            $options[5] = array(
                'attribute' => 'Token Key',
                'value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_token_key')
            );
            $options[6] = array(
                'attribute' => 'App Secret',
                'value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_app_secret')
            );

            $form['fieldset']['customerinfo'] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows' => $options
            );

            $license_header = array(
                'attribute' => array(
                    'data' => t('Attribute')
                ),
                'value' => array(
                    'data' => t('Value')
                )
            );

            $license_options = array();
            $license_options[0] = array(
                'attribute' => 'License Type',
                'value' => (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type') == '')? 'DEMO' : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type')
            );
            $license_options[1] = array(
                'attribute' => 'License Plan',
                'value' => (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_plan') == '')? 'DEMO' : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_plan')
            );
            $license_options[2] = array(
                'attribute' => 'No. of Users',
                'value' => (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_no_of_users') == '')? 1 : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_no_of_users')
            );

            $form['fieldset']['customer-license'] = array(
                '#theme' => 'table',
                '#header' => $license_header,
                '#rows' => $license_options,
                '#prefix' => '<div style="margin-top: 20px;"><h4>Your license info: </h4></div>'
            );
            $form['fecth_customer_license'] = array(
                '#type' => 'submit',
                '#value' => t('Check License'),
                '#submit' => array('::mo_auth_fetch_customer_license')
            );

            $form['main_layout_div_end'] = array(
                '#markup' => '<br><br></div>',
            );

            MoAuthUtilities::AddsupportTab( $form, $form_state);

            return $form;
        }

        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
        );

        $form['markup_14'] = array(
            '#markup' => '<h3>Register/Login with miniOrange</h3>'
        );

        $form['markup_15'] = array(
            '#markup' => '<div class="mo_saml_highlight_background_note_1">Just complete the short registration below to configure the Two-Factor Module. Please enter a valid email ID that you have access to. 
                          You will be able to move forward after verifying an OTP that we will send to this email.</div>'
        );

        $form['Mo_auth_customer_setup_username'] = array(
            '#type' => 'textfield',
            '#title' => t('Email<span style="color: red">*</span>'),
            '#attributes' => array(
                'autofocus' => 'true'
            ),
        );

        $form['Mo_auth_customer_setup_phone'] = array(
            '#type' => 'textfield',
            '#title' => t('Phone'),
            '#attributes' => array(
                'pattern' => '[\+]?[0-9]{1,4}\s?[0-9]{7,12}'
            ),
        );

        $form['markup_16'] = array(
            '#markup' => '<b>NOTE:</b> We will only call if you need support.'
        );

        $form['Mo_auth_customer_setup_password'] = array(
            '#type' => 'password_confirm',
        );

        $form['Mo_auth_customer_setup_button'] = array(
            '#type' => 'submit',
            '#value' => t('Register'),
            '#submit' => array('::Mo_auth_customer_setup_submit'),
        );

        $form['main_layout_div_end'] = array(
            '#markup' => '</div>',
        );

        MoAuthUtilities::AddsupportTab( $form, $form_state);

        return $form;
    }

    //Handle submit for customer setup.
    function Mo_auth_customer_setup_submit(&$form, $form_state) {

        $user = User::load(\Drupal::currentUser()->id());
        $user_id = $user->id();
        $username = $form['Mo_auth_customer_setup_username']['#value'];
        $phone = $form['Mo_auth_customer_setup_phone']['#value'];
        $password = $form['Mo_auth_customer_setup_password']['#value']['pass1'];

        if(empty($username)||empty($password)){
            drupal_set_message(t('The <b><u>Email</u></b> and <b><u>password</u></b> fields are mandatory.'), 'error');
            return;
        }elseif (!valid_email_address($username)) {
            drupal_set_message(t('The email address <b>' . $username . '</b> is not valid.'), 'error');
            return;
        }
        $customer_config = new MiniorangeCustomerSetup($username, $phone, $password, NULL);
        $check_customer_response = $customer_config->checkCustomer();
        if ($check_customer_response->status == 'CUSTOMER_NOT_FOUND') {
            // Create customer.
            // Store email and phone.
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_admin_email', $username)->save();
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_admin_phone', $phone)->save();
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_admin_password', $password)->save();
            $send_otp_response = $customer_config->sendOtp();
            if ($send_otp_response->status == 'SUCCESS') {
                // Store txID.
                \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_tx_id', $send_otp_response->txId)->save();
                $current_status = 'VALIDATE_OTP';
                \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_status', $current_status)->save();
                drupal_set_message(t('We have sent an OTP to @username. Please enter the OTP to verify your email.', array('@username' => $username)));
            }
            else if ($send_otp_response->status == 'FAILED') {
                drupal_set_message(t('Failed to send an OTP. Please check your internet connection'), 'error');
            }
        }
        elseif ($check_customer_response->status == 'CURL_ERROR') {
            drupal_set_message(t('cURL is not enabled. Please enable cURL'), 'error');
        }
        else {
            // Customer exists. Retrieve keys.
            $customer_keys_response = $customer_config->getCustomerKeys();
            if (json_last_error() == JSON_ERROR_NONE) {
                $this->mo_auth_save_customer($user_id, $customer_keys_response, $username, $phone);
                drupal_set_message(t('Your account has been retrieved successfully. Email Verification has been set as your default 2nd factor method.'));
            } else {
                drupal_set_message(t('Invalid credentials'), 'error');
            }
        }
    }

    // Validate OTP.
    function mo_auth_validate_otp_submit(&$form, $form_state)
    {
        $user = User::load(\Drupal::currentUser()->id());
        $user_id = $user->id();
        $otp_token = $form['mo_auth_customer_otp_token']['#value'];
        if (empty($otp_token)) {
            drupal_set_message(t('The <b>OTP</b> field is mandatory.'), 'error');
            return;
        }
        $username = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_email') == '')? NULL : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_email');
        $phone = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_phone') == '')? NULL : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_phone');
        $txId = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_tx_id') == '')? NULL : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_tx_id');
        $customerSetup = new MiniorangeCustomerSetup($username, $phone, NULL, $otp_token);

        // Validate OTP.
        $validate_otp_response = json_decode($customerSetup->validate_otp_token($txId, $otp_token, MoAuthConstants::$DEFAULT_CUSTOMER_ID, MoAuthConstants::$DEFAULT_CUSTOMER_API_KEY));
        if ($validate_otp_response->status == 'SUCCESS') {
            // OTP Validated. Show Configuration page.
            $current_status = 'PLUGIN_CONFIGURATION';
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_status', $current_status)->save();
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->clear('mo_auth_tx_id')->save();
            // OTP Validated. Create customer.
            $password = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_password');
            $customer_config = new MiniorangeCustomerSetup($username, $phone, $password, NULL);
            $create_customer_response = $customer_config->createCustomer();
            if ($create_customer_response->status == 'SUCCESS') {
                // Customer created.
                $this->mo_auth_save_customer($user_id, $create_customer_response, $username, $phone);
                drupal_set_message(t('Customer account created. Email Verification has been set as your default 2nd factor method.'));

            }else {
                drupal_set_message(t('An error occured while creating your account. Please try again or contact us at <a href="mailto:info@xecurify.com">info@xecurify.com</a>.'), 'error');
            }
        }
        else {
            drupal_set_message(t('The OTP you have entered is incorrect. Please try again.'), 'error');
        }
    }

    // Resend OTP.
    function mo_auth_resend_otp(&$form, $form_state)
    {
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->clear('mo_auth_tx_id')->save();
        $username = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_email');
        $phone = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_customer_admin_phone');
        $customer_config = new MiniorangeCustomerSetup($username, $phone, NULL, NULL);
        $send_otp_response = $customer_config->sendOtp();
        if ($send_otp_response->status == 'SUCCESS') {
            // Store txID.
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_tx_id', $send_otp_response->txId)->save();
            $current_status = 'VALIDATE_OTP';
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_status', $current_status)->save();
            drupal_set_message(t('We have sent an OTP to @username. Please enter the OTP to verify your email.', array('@username' => $username)));
        }
    }

    /**
     * Handle back button submit for customer setup.
     */
    function mo_auth_back($form, &$form_state)
    {
        $current_status = 'CUSTOMER_SETUP';
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_status', $current_status)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->clear('mo_auth_customer_admin_email')->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->clear('mo_auth_customer_admin_phone')->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->clear('mo_auth_tx_id')->save();
        drupal_set_message(t('Register/Login with your miniOrange Account'));
    }

    function mo_auth_save_customer($user_id, $json, $username, $phone)
    {
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_admin_email', $username)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_admin_phone', $phone)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_id', $json->id)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_api_key', $json->apiKey)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_token_key', $json->token)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_customer_app_secret', $json->appSecret)->save();

        $auth_method = AuthenticationType::$EMAIL_VERIFICATION['code'];
        $utilities = new MoAuthUtilities();
        $available = $utilities::check_for_userID($user_id);
        $database = \Drupal::database();
        $fields = array(
            'uid' => $user_id,
            'configured_auth_methods' => $auth_method,
            'miniorange_registered_email' => $username,
        );

        if($available == FALSE) {
            $database->insert('UserAuthenticationType')->fields($fields)->execute();
        }else {
            echo "error while saving authentication method."; exit;
        }

        $current_status = 'PLUGIN_CONFIGURATION';
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_status', $current_status)->save();

        // Update the customer second factor to OOB Email in miniOrange
        $customer = new MiniorangeCustomerProfile();
        $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $username, '', '', AuthenticationType::$EMAIL_VERIFICATION['code']);
        $user_api_handler = new UsersAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
        $user_api_handler->update($miniorange_user);

        $license_response = $user_api_handler->fetchLicense();

        $license_type = 'DEMO';
        $license_plan = 'DEMO';
        $no_of_users = 1;
        if ($license_response->status == 'SUCCESS') {

            $license_type = $license_response->licenseType;
            if ($license_type == 'DRUPAL_2FA_PLUGIN') {
                $license_plan = $license_response->licensePlan;
            }
            $no_of_users = $license_response->noOfUsers;
        }
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_type', $license_type)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_plan', $license_plan)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_no_of_users', $no_of_users)->save();

        $mo_auth_enable_two_factor = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_two_factor') == '' ? TRUE :\Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_two_factor');
        $mo_auth_enforce_inline_registration = \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enforce_inline_registration') == '' ? FALSE : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enforce_inline_registration');

        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enable_two_factor', $mo_auth_enable_two_factor)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enforce_inline_registration', $mo_auth_enforce_inline_registration)->save();
    }

    function mo_auth_fetch_customer_license($form, &$form_state)
    {
        $customer = new MiniorangeCustomerProfile();
        $user_api_handler = new UsersAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
        $response = $user_api_handler->fetchLicense();
        if ($response->status == 'SUCCESS') {
            $license_type = $response->licenseType;
            $license_plan = 'DEMO';
            if ($license_type == 'DRUPAL_2FA_PLUGIN') {
                $license_plan = $response->licensePlan;
            }
            $no_of_users = $response->noOfUsers;
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_type', $license_type)->save();
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_plan', $license_plan)->save();
            \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_2fa_license_no_of_users', $no_of_users)->save();
        }
        drupal_set_message(t('License fetched successfully.'), 'status');
    }

    function send_support_query($form, &$form_state) {
        $email = $form['miniorange_2fa_email_address_support']['#value'];
        $phone = $form['miniorange_2fa_phone_number_support']['#value'];
        $query = $form['miniorange_2fa_support_query_support']['#value'];
        MoAuthUtilities::send_support_query($email, $phone, $query);
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

    }
}