<?php
/**
 * @file
 * Contains support form for miniOrange 2FA Login Module.
 */
namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_2fa\MoAuthUtilities;

 /* Showing LoginSetting form info. */

class MoAuthLoginSettings extends FormBase
{    public function getFormId() {
        return 'miniorange_2fa_login_settings';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
        global $base_url ;
        $url = $base_url .'/admin/config/people/miniorange_2fa/licensing';

        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
        );

        $disabled = False;
        if (!MoAuthUtilities::isCustomerRegistered()) {
            $form['header'] = array(
                '#markup' => '<div class="mo_saml_register_message"><p>You need to <a href="'.$base_url.'/admin/config/people/miniorange_2fa/customer_setup">Register/Login</a> with miniOrange before using this module.</p></div><br>',
            );
            $disabled = True;
        }

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => 'miniorange_2fa/miniorange_2fa.admin',
            ),
        );

        $form['mo_auth_enable_two_factor'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_two_factor'),
            '#prefix' => '<div class="mo2f-setup-header mo2f-setup-header-top-left">Enable Two-Factor module</div><div class="mo2f-info">',
            '#suffix' => '</div><br>',
            '#title' => t('Enable Two-Factor plugin. <span style="color:red;">(If you disable this checkbox, Second-Factor authentication will not be invoked for any user during login.)</span>'),
            '#disabled' => $disabled,
        );

        $license_type = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type') == '')? 'DEMO' : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type');
        $is_free = $license_type == 'DRUPAL_2FA_PLUGIN' || $license_type == 'PREMIUM' ? FALSE : TRUE;
        $is_free_class = $is_free ? " mo2f-grayed-out" : "";

        $form['mo_auth_enforce_inline_registration'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enforce_inline_registration'),
            '#prefix' => '<div class="mo2f-setup-header mo2f-setup-header-top-left">Invoke Inline Registration to setup 2nd factor for users <a href= "'.$url.'"><b>[PREMIUM]</b></a></span></div><div class="mo2f-info ' . $is_free_class . '">',
            '#suffix' => '</div><br><br>',
            '#disabled' => $is_free,
            '#title' => t('Enforce 2 Factor registration for users at login time.'),
        );

        $form['Submit_LoginSettings_form'] = array(
            '#type' => 'submit',
            '#value' => t('Save Settings'),
            '#disabled' => $disabled,
        );

        $form['main_layout_div_end'] = array(
            '#markup' => '<br><br><br><br><br><br><br><br><br><br><br><br><br></div>',
        );
        MoAuthUtilities::AddsupportTab( $form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $two_factor = $form['mo_auth_enable_two_factor']['#value'];
        $inline_registration = $form['mo_auth_enforce_inline_registration']['#value'];

        $enable2Factor = FALSE;
        $enforceInlineRegistration = FALSE;

        if ($two_factor == 1) {
            $enable2Factor = TRUE;
        }
        if ($inline_registration == 1) {
            $enforceInlineRegistration = TRUE;
        }

        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enable_two_factor', $enable2Factor)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enforce_inline_registration', $enforceInlineRegistration)->save();

        drupal_set_message(t("Login settings updated."), 'status');
    }

    function send_support_query($form, &$form_state)
    {
        $email = $form['miniorange_2fa_email_address_support']['#value'];
        $phone = $form['miniorange_2fa_phone_number_support']['#value'];
        $query = $form['miniorange_2fa_support_query_support']['#value'];
        MoAuthUtilities::send_support_query($email, $phone, $query);
    }
}